@@ -46,11 +46,11 @@ QJsonObject readOptionalInfo(const QString &suffix)
     return doc.object();
 }
 
-void setOptionalInfo(QImage& image, const QString &suffix)
+void setOptionalInfo(QImage &image, const QString &suffix)
 {
     auto obj = readOptionalInfo(suffix);
     if (obj.isEmpty()) {
-        return ;
+        return;
     }
 
     // Set resolution
@@ -70,7 +70,16 @@ void setOptionalInfo(QImage& image, const QString &suffix)
     }
 }
 
-bool checkOptionalInfo(QImage& image, const QString &suffix)
+QByteArray readSubType(const QString &suffix)
+{
+    auto obj = readOptionalInfo(suffix);
+    if (obj.isEmpty()) {
+        return {};
+    }
+    return obj.value("subType").toString().toLatin1();
+}
+
+bool checkOptionalInfo(QImage &image, const QString &suffix)
 {
     auto obj = readOptionalInfo(suffix);
     if (obj.isEmpty()) {
@@ -157,6 +166,9 @@ int basicTest(const QString &suffix, bool lossless, bool ignoreDataCheck, bool s
         {
             QBuffer buffer(&writtenData);
             QImageWriter imgWriter(&buffer, format.constData());
+            auto subType = readSubType(suffix);
+            if (!subType.isEmpty())
+                imgWriter.setSubType(subType);
             if (lossless) {
                 imgWriter.setQuality(100);
             }
@@ -258,7 +270,6 @@ QImage formatSourceImage(const QImage::Format &format)
     auto folder = QStringLiteral("%1/format/_images").arg(IMAGEDIR);
 
     switch (format) {
-
     case QImage::Format_MonoLSB:
     case QImage::Format_Mono:
         image = QImage(QStringLiteral("%1/mono.png").arg(folder));
@@ -371,7 +382,12 @@ int formatTest(const QString &suffix, bool createTemplates)
         QBuffer buffer(&ba);
         auto writtenImage = QImageReader(&buffer, suffix.toLatin1()).read();
         if (writtenImage.isNull()) {
-            ++failed;
+            if (suffix.toLatin1() == "heif") {
+                // libheif + ffmpeg decoder is unable to load all HEIF files.
+                ++skipped;
+            } else {
+                ++failed;
+            }
             QTextStream(stdout) << "FAIL : error while reading the image " << formatName << "\n";
             continue;
         }
@@ -403,8 +419,16 @@ int formatTest(const QString &suffix, bool createTemplates)
         // checking the format: must be the same
         if (writtenImage.format() != tmplImage.format()) {
             ++failed;
+            auto writtenformatName = QString(QMetaEnum::fromType<QImage::Format>().valueToKey(writtenImage.format()));
             auto tmplformatName = QString(QMetaEnum::fromType<QImage::Format>().valueToKey(tmplImage.format()));
-            QTextStream(stdout) << "FAIL : format mismatch " << formatName << " != " << tmplformatName << "\n";
+            QTextStream(stdout) << "FAIL : format mismatch " << formatName << " (";
+            if (format == writtenImage.format()) {
+                QTextStream(stdout) << "template image: " << tmplformatName << ")\n";
+            } else if (format == tmplImage.format()) {
+                QTextStream(stdout) << "written image: " << writtenformatName << ")\n";
+            } else {
+                QTextStream(stdout) << writtenformatName << " != " << tmplformatName << ")\n";
+            }
             continue;
         }
 
@@ -554,7 +578,7 @@ int nullDeviceTest(const QString &suffix)
     writer.optimizedWrite();
     writer.progressiveScanWrite();
 
-    if (failed == 0) {// success
+    if (failed == 0) { // success
         ++passed;
     }
 
@@ -586,8 +610,7 @@ int main(int argc, char **argv)
                             QStringLiteral("max"));
     QCommandLineOption createFormatTempates({QStringLiteral("create-format-templates")},
                                             QStringLiteral("Create template images for all formats supported by QImage."));
-    QCommandLineOption skipOptTest({QStringLiteral("skip-optional-tests")},
-                                   QStringLiteral("Skip optional data tests (metadata, resolution, etc.)."));
+    QCommandLineOption skipOptTest({QStringLiteral("skip-optional-tests")}, QStringLiteral("Skip optional data tests (metadata, resolution, etc.)."));
 
     parser.addOption(lossless);
     parser.addOption(ignoreDataCheck);
@@ -616,8 +639,28 @@ int main(int argc, char **argv)
         }
     }
 
-    // run test
     auto suffix = args.at(0);
+
+    // skip test if libheif configuration is obviously incomplete
+    QByteArray format = suffix.toLatin1();
+    const QList<QByteArray> read_formats = QImageReader::supportedImageFormats();
+    const QList<QByteArray> write_formats = QImageWriter::supportedImageFormats();
+
+    if (!read_formats.contains(format)) {
+        if (format == "heif" || format == "hej2") {
+            QTextStream(stdout) << "WARNING : libheif configuration is missing necessary decoder(s)!\n";
+            return 0;
+        }
+    }
+
+    if (!write_formats.contains(format)) {
+        if (format == "heif" || format == "hej2") {
+            QTextStream(stdout) << "WARNING : libheif configuration is missing necessary encoder(s)!\n";
+            return 0;
+        }
+    }
+
+    // run test
     auto ret = basicTest(suffix, parser.isSet(lossless), parser.isSet(ignoreDataCheck), parser.isSet(skipOptTest), fuzzarg);
     if (ret == 0) {
         ret = formatTest(suffix, parser.isSet(createFormatTempates));
