@@ -8,21 +8,37 @@
 */
 
 #include "heif_p.h"
+#include "microexif_p.h"
 #include "util_p.h"
 #include <libheif/heif.h>
 
 #include <QColorSpace>
-#include <QDebug>
+#include <QLoggingCategory>
 #include <QPointF>
 #include <QSysInfo>
+
+#include <cstring>
 #include <limits>
-#include <string.h>
+
+#ifdef QT_DEBUG
+Q_LOGGING_CATEGORY(LOG_HEIFPLUGIN, "kf.imageformats.plugins.heif", QtDebugMsg)
+#else
+Q_LOGGING_CATEGORY(LOG_HEIFPLUGIN, "kf.imageformats.plugins.heif", QtWarningMsg)
+#endif
+
+#ifndef HEIF_MAX_METADATA_SIZE
+/*!
+ * XMP and EXIF maximum size.
+ */
+#define HEIF_MAX_METADATA_SIZE (4 * 1024 * 1024)
+#endif
 
 size_t HEIFHandler::m_initialized_count = 0;
 bool HEIFHandler::m_plugins_queried = false;
 bool HEIFHandler::m_heif_decoder_available = false;
 bool HEIFHandler::m_heif_encoder_available = false;
 bool HEIFHandler::m_hej2_decoder_available = false;
+bool HEIFHandler::m_hej2_encoder_available = false;
 bool HEIFHandler::m_avci_decoder_available = false;
 
 extern "C" {
@@ -103,7 +119,7 @@ bool HEIFHandler::read(QImage *outImage)
 bool HEIFHandler::write(const QImage &image)
 {
     if (image.format() == QImage::Format_Invalid || image.isNull()) {
-        qWarning("No image data to save");
+        qCWarning(LOG_HEIFPLUGIN) << "No image data to save";
         return false;
     }
 
@@ -146,6 +162,14 @@ bool HEIFHandler::write_helper(const QImage &image)
         break;
     }
 
+    heif_compression_format encoder_codec = heif_compression_HEVC;
+#if LIBHEIF_HAVE_VERSION(1, 13, 0)
+    if (format() == "hej2") {
+        encoder_codec = heif_compression_JPEG2000;
+        save_depth = 8; // for compatibility reasons
+    }
+#endif
+
     heif_chroma chroma;
     if (save_depth > 8) {
         if (save_alpha) {
@@ -191,7 +215,7 @@ bool HEIFHandler::write_helper(const QImage &image)
 
     err = heif_image_create(tmpimage.width(), tmpimage.height(), heif_colorspace_RGB, chroma, &h_image);
     if (err.code) {
-        qWarning() << "heif_image_create error:" << err.message;
+        qCWarning(LOG_HEIFPLUGIN) << "heif_image_create error:" << err.message;
         heif_context_free(context);
         return false;
     }
@@ -270,7 +294,7 @@ bool HEIFHandler::write_helper(const QImage &image)
         }
         break;
     default:
-        qWarning() << "Unsupported depth:" << save_depth;
+        qCWarning(LOG_HEIFPLUGIN) << "Unsupported depth:" << save_depth;
         heif_image_release(h_image);
         heif_context_free(context);
         return false;
@@ -278,9 +302,9 @@ bool HEIFHandler::write_helper(const QImage &image)
     }
 
     struct heif_encoder *encoder = nullptr;
-    err = heif_context_get_encoder_for_format(context, heif_compression_HEVC, &encoder);
+    err = heif_context_get_encoder_for_format(context, encoder_codec, &encoder);
     if (err.code) {
-        qWarning() << "Unable to get an encoder instance:" << err.message;
+        qCWarning(LOG_HEIFPLUGIN) << "Unable to get an encoder instance:" << err.message;
         heif_image_release(h_image);
         heif_context_free(context);
         return false;
@@ -298,21 +322,39 @@ bool HEIFHandler::write_helper(const QImage &image)
     encoder_options->save_alpha_channel = save_alpha;
 
     if ((tmpimage.width() % 2 == 1) || (tmpimage.height() % 2 == 1)) {
-        qWarning() << "Image has odd dimension!\nUse even-numbered dimension(s) for better compatibility with other HEIF implementations.";
+        qCWarning(LOG_HEIFPLUGIN) << "Image has odd dimension!\nUse even-numbered dimension(s) for better compatibility with other HEIF implementations.";
         if (save_alpha) {
             // This helps to save alpha channel when image has odd dimension
             encoder_options->macOS_compatibility_workaround = 0;
         }
     }
 
-    err = heif_context_encode_image(context, h_image, encoder, encoder_options, nullptr);
+    struct heif_image_handle *handle;
+    err = heif_context_encode_image(context, h_image, encoder, encoder_options, &handle);
+
+    // exif metadata
+    if (err.code == heif_error_Ok) {
+        auto exif = MicroExif::fromImage(tmpimage);
+        if (!exif.isEmpty()) {
+            auto ba = exif.toByteArray();
+            err = heif_context_add_exif_metadata(context, handle, ba.constData(), ba.size());
+        }
+    }
+    // xmp metadata
+    if (err.code == heif_error_Ok) {
+        auto xmp = image.text(QStringLiteral(META_KEY_XMP_ADOBE));
+        if (!xmp.isEmpty()) {
+            auto ba = xmp.toUtf8();
+            err = heif_context_add_XMP_metadata(context, handle, ba.constData(), ba.size());
+        }
+    }
 
     if (encoder_options) {
         heif_encoding_options_free(encoder_options);
     }
 
     if (err.code) {
-        qWarning() << "heif_context_encode_image failed:" << err.message;
+        qCWarning(LOG_HEIFPLUGIN) << "heif_context_encode_image failed:" << err.message;
         heif_encoder_release(encoder);
         heif_image_release(h_image);
         heif_context_free(context);
@@ -329,7 +371,7 @@ bool HEIFHandler::write_helper(const QImage &image)
     heif_image_release(h_image);
 
     if (err.code) {
-        qWarning() << "Writing HEIF image failed:" << err.message;
+        qCWarning(LOG_HEIFPLUGIN) << "Writing HEIF image failed:" << err.message;
         heif_context_free(context);
         return false;
     }
@@ -489,7 +531,7 @@ bool HEIFHandler::ensureDecoder()
     struct heif_error err = heif_context_read_from_memory(ctx, static_cast<const void *>(buffer.constData()), buffer.size(), nullptr);
 
     if (err.code) {
-        qWarning() << "heif_context_read_from_memory error:" << err.message;
+        qCWarning(LOG_HEIFPLUGIN) << "heif_context_read_from_memory error:" << err.message;
         heif_context_free(ctx);
         m_parseState = ParseHeicError;
         return false;
@@ -498,7 +540,7 @@ bool HEIFHandler::ensureDecoder()
     struct heif_image_handle *handle = nullptr;
     err = heif_context_get_primary_image_handle(ctx, &handle);
     if (err.code) {
-        qWarning() << "heif_context_get_primary_image_handle error:" << err.message;
+        qCWarning(LOG_HEIFPLUGIN) << "heif_context_get_primary_image_handle error:" << err.message;
         heif_context_free(ctx);
         m_parseState = ParseHeicError;
         return false;
@@ -508,7 +550,7 @@ bool HEIFHandler::ensureDecoder()
         m_parseState = ParseHeicError;
         heif_image_handle_release(handle);
         heif_context_free(ctx);
-        qWarning() << "HEIC image has zero dimension";
+        qCWarning(LOG_HEIFPLUGIN) << "HEIC image has zero dimension";
         return false;
     }
 
@@ -518,7 +560,7 @@ bool HEIFHandler::ensureDecoder()
         m_parseState = ParseHeicError;
         heif_image_handle_release(handle);
         heif_context_free(ctx);
-        qWarning() << "HEIF image with undefined or unsupported bit depth.";
+        qCWarning(LOG_HEIFPLUGIN) << "HEIF image with undefined or unsupported bit depth.";
         return false;
     }
 
@@ -527,7 +569,7 @@ bool HEIFHandler::ensureDecoder()
 
     QImage::Format target_image_format;
 
-    if (bit_depth == 10 || bit_depth == 12) {
+    if (bit_depth == 10 || bit_depth == 12 || bit_depth == 16) {
         if (hasAlphaChannel) {
             chroma = (QSysInfo::ByteOrder == QSysInfo::LittleEndian) ? heif_chroma_interleaved_RRGGBBAA_LE : heif_chroma_interleaved_RRGGBBAA_BE;
             target_image_format = QImage::Format_RGBA64;
@@ -547,7 +589,7 @@ bool HEIFHandler::ensureDecoder()
         m_parseState = ParseHeicError;
         heif_image_handle_release(handle);
         heif_context_free(ctx);
-        qWarning() << "Unsupported bit depth:" << bit_depth;
+        qCWarning(LOG_HEIFPLUGIN) << "Unsupported bit depth:" << bit_depth;
         return false;
     }
 
@@ -562,7 +604,7 @@ bool HEIFHandler::ensureDecoder()
 
 #if LIBHEIF_HAVE_VERSION(1, 13, 0)
     if (err.code == heif_error_Invalid_input && err.subcode == heif_suberror_Unknown_NCLX_matrix_coefficients && img == nullptr && buffer.contains("Xiaomi")) {
-        qWarning() << "Non-standard HEIF image with invalid matrix_coefficients, probably made by a Xiaomi device!";
+        qCWarning(LOG_HEIFPLUGIN) << "Non-standard HEIF image with invalid matrix_coefficients, probably made by a Xiaomi device!";
 
         // second try to decode with strict decoding disabled
         decoder_option->strict_decoding = 0;
@@ -575,7 +617,7 @@ bool HEIFHandler::ensureDecoder()
     }
 
     if (err.code) {
-        qWarning() << "heif_decode_image error:" << err.message;
+        qCWarning(LOG_HEIFPLUGIN) << "heif_decode_image error:" << err.message;
         heif_image_handle_release(handle);
         heif_context_free(ctx);
         m_parseState = ParseHeicError;
@@ -592,7 +634,7 @@ bool HEIFHandler::ensureDecoder()
         heif_image_handle_release(handle);
         heif_context_free(ctx);
         m_parseState = ParseHeicError;
-        qWarning() << "HEIC image size invalid:" << imageSize;
+        qCWarning(LOG_HEIFPLUGIN) << "HEIC image size invalid:" << imageSize;
         return false;
     }
 
@@ -604,7 +646,7 @@ bool HEIFHandler::ensureDecoder()
         heif_image_handle_release(handle);
         heif_context_free(ctx);
         m_parseState = ParseHeicError;
-        qWarning() << "HEIC data pixels information not valid!";
+        qCWarning(LOG_HEIFPLUGIN) << "HEIC data pixels information not valid!";
         return false;
     }
 
@@ -614,11 +656,40 @@ bool HEIFHandler::ensureDecoder()
         heif_image_handle_release(handle);
         heif_context_free(ctx);
         m_parseState = ParseHeicError;
-        qWarning() << "Unable to allocate memory!";
+        qCWarning(LOG_HEIFPLUGIN) << "Unable to allocate memory!";
         return false;
     }
 
     switch (bit_depth) {
+    case 16:
+        if (hasAlphaChannel) {
+            for (int y = 0; y < imageHeight; y++) {
+                memcpy(m_current_image.scanLine(y), src + (y * stride), 8 * size_t(imageWidth));
+            }
+        } else { // no alpha channel
+            for (int y = 0; y < imageHeight; y++) {
+                const uint16_t *src_word = reinterpret_cast<const uint16_t *>(src + (y * stride));
+                uint16_t *dest_data = reinterpret_cast<uint16_t *>(m_current_image.scanLine(y));
+                for (int x = 0; x < imageWidth; x++) {
+                    // R
+                    *dest_data = *src_word;
+                    src_word++;
+                    dest_data++;
+                    // G
+                    *dest_data = *src_word;
+                    src_word++;
+                    dest_data++;
+                    // B
+                    *dest_data = *src_word;
+                    src_word++;
+                    dest_data++;
+                    // X = 0xffff
+                    *dest_data = 0xffff;
+                    dest_data++;
+                }
+            }
+        }
+        break;
     case 12:
         if (hasAlphaChannel) {
             for (int y = 0; y < imageHeight; y++) {
@@ -780,7 +851,7 @@ bool HEIFHandler::ensureDecoder()
         heif_image_handle_release(handle);
         heif_context_free(ctx);
         m_parseState = ParseHeicError;
-        qWarning() << "Unsupported bit depth:" << bit_depth;
+        qCWarning(LOG_HEIFPLUGIN) << "Unsupported bit depth:" << bit_depth;
         return false;
         break;
     }
@@ -792,22 +863,52 @@ bool HEIFHandler::ensureDecoder()
             QByteArray ba(rawProfileSize, 0);
             err = heif_image_handle_get_raw_color_profile(handle, ba.data());
             if (err.code) {
-                qWarning() << "icc profile loading failed";
+                qCWarning(LOG_HEIFPLUGIN) << "icc profile loading failed";
             } else {
-                m_current_image.setColorSpace(QColorSpace::fromIccProfile(ba));
-                if (!m_current_image.colorSpace().isValid()) {
-                    qWarning() << "HEIC image has Qt-unsupported or invalid ICC profile!";
+                QColorSpace colorspace = QColorSpace::fromIccProfile(ba);
+                if (!colorspace.isValid()) {
+                    qCWarning(LOG_HEIFPLUGIN) << "HEIC image has Qt-unsupported or invalid ICC profile!";
                 }
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 8, 0))
+                else if (colorspace.colorModel() == QColorSpace::ColorModel::Cmyk) {
+                    qCWarning(LOG_HEIFPLUGIN) << "CMYK ICC profile is not expected for HEIF, discarding the ICCprofile!";
+                    colorspace = QColorSpace();
+                } else if (colorspace.colorModel() == QColorSpace::ColorModel::Gray) {
+                    if (hasAlphaChannel) {
+                        QPointF gray_whitePoint = colorspace.whitePoint();
+                        if (gray_whitePoint.isNull()) {
+                            gray_whitePoint = QPointF(0.3127f, 0.329f);
+                        }
+
+                        const QPointF redP(0.64f, 0.33f);
+                        const QPointF greenP(0.3f, 0.6f);
+                        const QPointF blueP(0.15f, 0.06f);
+
+                        QColorSpace::TransferFunction trc_new = colorspace.transferFunction();
+                        float gamma_new = colorspace.gamma();
+                        if (trc_new == QColorSpace::TransferFunction::Custom) {
+                            trc_new = QColorSpace::TransferFunction::SRgb;
+                        }
+                        colorspace = QColorSpace(gray_whitePoint, redP, greenP, blueP, trc_new, gamma_new);
+                        if (!colorspace.isValid()) {
+                            qCWarning(LOG_HEIFPLUGIN) << "HEIF plugin created invalid QColorSpace!";
+                        }
+                    } else { // no alpha channel
+                        m_current_image.convertTo(bit_depth > 8 ? QImage::Format_Grayscale16 : QImage::Format_Grayscale8);
+                    }
+                }
+#endif
+                m_current_image.setColorSpace(colorspace);
             }
         } else {
-            qWarning() << "icc profile is empty or above limits";
+            qCWarning(LOG_HEIFPLUGIN) << "icc profile is empty or above limits";
         }
 
     } else if (profileType == heif_color_profile_type_nclx) {
         struct heif_color_profile_nclx *nclx = nullptr;
         err = heif_image_handle_get_nclx_color_profile(handle, &nclx);
         if (err.code || !nclx) {
-            qWarning() << "nclx profile loading failed";
+            qCWarning(LOG_HEIFPLUGIN) << "nclx profile loading failed";
         } else {
             const QPointF redPoint(nclx->color_primary_red_x, nclx->color_primary_red_y);
             const QPointF greenPoint(nclx->color_primary_green_x, nclx->color_primary_green_y);
@@ -842,9 +943,9 @@ bool HEIFHandler::ensureDecoder()
                 break;
 #endif
             default:
-                qWarning("CICP color_primaries: %d, transfer_characteristics: %d\nThe colorspace is unsupported by this plug-in yet.",
-                         nclx->color_primaries,
-                         nclx->transfer_characteristics);
+                qCWarning(LOG_HEIFPLUGIN) << "CICP color_primaries: %d, transfer_characteristics: %d\nThe colorspace is unsupported by this plug-in yet."
+                                          << nclx->color_primaries
+                                          << nclx->transfer_characteristics;
                 q_trc = QColorSpace::TransferFunction::SRgb;
                 break;
             }
@@ -866,7 +967,7 @@ bool HEIFHandler::ensureDecoder()
             heif_nclx_color_profile_free(nclx);
 
             if (!m_current_image.colorSpace().isValid()) {
-                qWarning() << "HEIC plugin created invalid QColorSpace from NCLX!";
+                qCWarning(LOG_HEIFPLUGIN) << "HEIC plugin created invalid QColorSpace from NCLX!";
             }
         }
 
@@ -874,6 +975,38 @@ bool HEIFHandler::ensureDecoder()
         m_current_image.setColorSpace(QColorSpace(QColorSpace::SRgb));
     }
 
+    // read metadata
+    if (auto numMetadata = heif_image_handle_get_number_of_metadata_blocks(handle, nullptr)) {
+        QVector<heif_item_id> ids(numMetadata);
+        heif_image_handle_get_list_of_metadata_block_IDs(handle, nullptr, ids.data(), numMetadata);
+        for (int n = 0; n < numMetadata; ++n) {
+            auto itemtype = heif_image_handle_get_metadata_type(handle, ids[n]);
+            auto contenttype = heif_image_handle_get_metadata_content_type(handle, ids[n]);
+            auto isExif = !std::strcmp(itemtype, "Exif");
+            auto isXmp = !std::strcmp(contenttype, "application/rdf+xml");
+            if (isExif || isXmp) {
+                auto sz = heif_image_handle_get_metadata_size(handle, ids[n]);
+                if (sz == 0 || sz >= HEIF_MAX_METADATA_SIZE)
+                    continue;
+                QByteArray ba(sz, char());
+                auto err = heif_image_handle_get_metadata(handle, ids[n], ba.data());
+                if (err.code != heif_error_Ok) {
+                    qCWarning(LOG_HEIFPLUGIN) << "Error while reading metadata" << err.message;
+                    continue;
+                }
+                if (isXmp) {
+                    m_current_image.setText(QStringLiteral(META_KEY_XMP_ADOBE), QString::fromUtf8(ba));
+                } else if (isExif) {
+                    auto exif = MicroExif::fromByteArray(ba, true);
+                    if (!exif.isEmpty()) {
+                        exif.updateImageResolution(m_current_image);
+                        exif.updateImageMetadata(m_current_image);
+                    }
+                }
+            }
+        }
+    }
+
     heif_image_release(img);
     heif_image_handle_release(handle);
     heif_context_free(ctx);
@@ -902,6 +1035,13 @@ bool HEIFHandler::isHej2DecoderAvailable()
     return m_hej2_decoder_available;
 }
 
+bool HEIFHandler::isHej2EncoderAvailable()
+{
+    HEIFHandler::queryHeifLib();
+
+    return m_hej2_encoder_available;
+}
+
 bool HEIFHandler::isAVCIDecoderAvailable()
 {
     HEIFHandler::queryHeifLib();
@@ -924,8 +1064,9 @@ void HEIFHandler::queryHeifLib()
         m_heif_decoder_available = heif_have_decoder_for_format(heif_compression_HEVC);
 #if LIBHEIF_HAVE_VERSION(1, 13, 0)
         m_hej2_decoder_available = heif_have_decoder_for_format(heif_compression_JPEG2000);
+        m_hej2_encoder_available = heif_have_encoder_for_format(heif_compression_JPEG2000);
 #endif
-#if LIBHEIF_HAVE_VERSION(1, 19, 0)
+#if LIBHEIF_HAVE_VERSION(1, 19, 6)
         m_avci_decoder_available = heif_have_decoder_for_format(heif_compression_AVC);
 #endif
         m_plugins_queried = true;
@@ -992,6 +1133,9 @@ QImageIOPlugin::Capabilities HEIFPlugin::capabilities(QIODevice *device, const Q
         if (HEIFHandler::isHej2DecoderAvailable()) {
             format_cap |= CanRead;
         }
+        if (HEIFHandler::isHej2EncoderAvailable()) {
+            format_cap |= CanWrite;
+        }
         return format_cap;
     }
 
@@ -1021,7 +1165,7 @@ QImageIOPlugin::Capabilities HEIFPlugin::capabilities(QIODevice *device, const Q
         }
     }
 
-    if (device->isWritable() && HEIFHandler::isHeifEncoderAvailable()) {
+    if (device->isWritable() && (HEIFHandler::isHeifEncoderAvailable() || HEIFHandler::isHej2EncoderAvailable())) {
         cap |= CanWrite;
     }
     return cap;
