@@ -6,12 +6,20 @@
 */
 
 #include "pcx_p.h"
+#include "scanlineconverter_p.h"
 #include "util_p.h"
 
 #include <QColor>
+#include <QColorSpace>
 #include <QDataStream>
-#include <QDebug>
 #include <QImage>
+#include <QLoggingCategory>
+
+#ifdef QT_DEBUG
+Q_LOGGING_CATEGORY(LOG_PCXPLUGIN, "kf.imageformats.plugins.pcx", QtDebugMsg)
+#else
+Q_LOGGING_CATEGORY(LOG_PCXPLUGIN, "kf.imageformats.plugins.pcx", QtWarningMsg)
+#endif
 
 #pragma pack(push, 1)
 class RGB
@@ -92,12 +100,18 @@ public:
             fmt = QImage::Format_Mono;
         } else if (Bpp == 1 && NPlanes == 4) {
             fmt = QImage::Format_Indexed8;
+        } else if (Bpp == 1 && NPlanes == 3) {
+            fmt = QImage::Format_Indexed8;
         } else if (Bpp == 4 && NPlanes == 1) {
             fmt = QImage::Format_Indexed8;
+        } else if (Bpp == 2 && NPlanes == 1) {
+            fmt = QImage::Format_Indexed8;
         } else if (Bpp == 8 && NPlanes == 1) {
             fmt = QImage::Format_Indexed8;
         } else if (Bpp == 8 && NPlanes == 3) {
             fmt = QImage::Format_RGB32;
+        } else if (Bpp == 8 && NPlanes == 4) {
+            fmt = QImage::Format_ARGB32;
         }
         return fmt;
     }
@@ -317,7 +331,7 @@ static bool readImage1(QImage &img, QDataStream &s, const PCXHEADER &header)
     img.setColorCount(2);
 
     if (img.isNull()) {
-        qWarning() << "Failed to allocate image, invalid dimensions?" << QSize(header.width(), header.height());
+        qCWarning(LOG_PCXPLUGIN) << "Failed to allocate image, invalid dimensions?" << QSize(header.width(), header.height());
         return false;
     }
 
@@ -346,18 +360,18 @@ static bool readImage1(QImage &img, QDataStream &s, const PCXHEADER &header)
 
 static bool readImage4(QImage &img, QDataStream &s, const PCXHEADER &header)
 {
-    QByteArray buf(header.BytesPerLine * 4, 0);
+    QByteArray buf(header.BytesPerLine * header.NPlanes, 0);
     QByteArray pixbuf(header.width(), 0);
 
     img = imageAlloc(header.width(), header.height(), header.format());
     img.setColorCount(16);
     if (img.isNull()) {
-        qWarning() << "Failed to allocate image, invalid dimensions?" << QSize(header.width(), header.height());
+        qCWarning(LOG_PCXPLUGIN) << "Failed to allocate image, invalid dimensions?" << QSize(header.width(), header.height());
         return false;
     }
 
     if (header.BytesPerLine < (header.width() + 7) / 8) {
-        qWarning() << "PCX image has invalid BytesPerLine value";
+        qCWarning(LOG_PCXPLUGIN) << "PCX image has invalid BytesPerLine value";
         return false;
     }
 
@@ -371,7 +385,7 @@ static bool readImage4(QImage &img, QDataStream &s, const PCXHEADER &header)
             return false;
         }
 
-        for (int i = 0; i < 4; i++) {
+        for (int i = 0; i < header.NPlanes; i++) {
             quint32 offset = i * header.BytesPerLine;
             for (int x = 0; x < header.width(); ++x) {
                 if (buf[offset + (x / 8)] & (128 >> (x % 8))) {
@@ -382,7 +396,7 @@ static bool readImage4(QImage &img, QDataStream &s, const PCXHEADER &header)
 
         uchar *p = img.scanLine(y);
         if (!p) {
-            qWarning() << "Failed to get scanline for" << y << "might be out of bounds";
+            qCWarning(LOG_PCXPLUGIN) << "Failed to get scanline for" << y << "might be out of bounds";
         }
         for (int x = 0; x < header.width(); ++x) {
             p[x] = pixbuf[x];
@@ -397,6 +411,49 @@ static bool readImage4(QImage &img, QDataStream &s, const PCXHEADER &header)
     return true;
 }
 
+static bool readImage2(QImage &img, QDataStream &s, const PCXHEADER &header)
+{
+    QByteArray buf(header.BytesPerLine, 0);
+
+    img = imageAlloc(header.width(), header.height(), header.format());
+    img.setColorCount(4);
+
+    if (img.isNull()) {
+        qCWarning(LOG_PCXPLUGIN) << "Failed to allocate image, invalid dimensions?" << QSize(header.width(), header.height());
+        return false;
+    }
+
+    for (int y = 0; y < header.height(); ++y) {
+        if (s.atEnd()) {
+            return false;
+        }
+
+        if (!readLine(s, buf, header)) {
+            return false;
+        }
+
+        uchar *p = img.scanLine(y);
+        if (!p) {
+            return false;
+        }
+
+        const unsigned int bpl = std::min(header.BytesPerLine, static_cast<quint16>(header.width() / 4));
+        for (unsigned int x = 0; x < bpl; ++x) {
+            p[x * 4] = (buf[x] >> 6) & 3;
+            p[x * 4 + 1] = (buf[x] >> 4) & 3;
+            p[x * 4 + 2] = (buf[x] >> 2) & 3;
+            p[x * 4 + 3] = buf[x] & 3;
+        }
+    }
+
+    // Read the palette
+    for (int i = 0; i < 4; ++i) {
+        img.setColor(i, header.ColorMap.color(i));
+    }
+
+    return (s.status() == QDataStream::Ok);
+}
+
 static bool readImage4v2(QImage &img, QDataStream &s, const PCXHEADER &header)
 {
     QByteArray buf(header.BytesPerLine, 0);
@@ -405,7 +462,7 @@ static bool readImage4v2(QImage &img, QDataStream &s, const PCXHEADER &header)
     img.setColorCount(16);
 
     if (img.isNull()) {
-        qWarning() << "Failed to allocate image, invalid dimensions?" << QSize(header.width(), header.height());
+        qCWarning(LOG_PCXPLUGIN) << "Failed to allocate image, invalid dimensions?" << QSize(header.width(), header.height());
         return false;
     }
 
@@ -446,7 +503,7 @@ static bool readImage8(QImage &img, QDataStream &s, const PCXHEADER &header)
     img.setColorCount(256);
 
     if (img.isNull()) {
-        qWarning() << "Failed to allocate image, invalid dimensions?" << QSize(header.width(), header.height());
+        qCWarning(LOG_PCXPLUGIN) << "Failed to allocate image, invalid dimensions?" << QSize(header.width(), header.height());
         return false;
     }
 
@@ -484,7 +541,6 @@ static bool readImage8(QImage &img, QDataStream &s, const PCXHEADER &header)
         }
     }
 
-    //   qDebug() << "Palette Flag: " << flag;
     if (flag == 12 && (header.Version == 5 || header.Version == 2)) {
         // Read the palette
         quint8 r;
@@ -504,11 +560,12 @@ static bool readImage24(QImage &img, QDataStream &s, const PCXHEADER &header)
     QByteArray r_buf(header.BytesPerLine, 0);
     QByteArray g_buf(header.BytesPerLine, 0);
     QByteArray b_buf(header.BytesPerLine, 0);
+    QByteArray a_buf(header.BytesPerLine, char(0xFF));
 
     img = imageAlloc(header.width(), header.height(), header.format());
 
     if (img.isNull()) {
-        qWarning() << "Failed to allocate image, invalid dimensions?" << QSize(header.width(), header.height());
+        qCWarning(LOG_PCXPLUGIN) << "Failed to allocate image, invalid dimensions?" << QSize(header.width(), header.height());
         return false;
     }
 
@@ -528,11 +585,13 @@ static bool readImage24(QImage &img, QDataStream &s, const PCXHEADER &header)
         if (!readLine(s, b_buf, header)) {
             return false;
         }
+        if (header.NPlanes == 4 && !readLine(s, a_buf, header)) {
+            return false;
+        }
 
-        uint *p = (uint *)img.scanLine(y);
-
+        auto p = reinterpret_cast<QRgb *>(img.scanLine(y));
         for (unsigned int x = 0; x < bpl; ++x) {
-            p[x] = qRgb(r_buf[x], g_buf[x], b_buf[x]);
+            p[x] = qRgba(r_buf[x], g_buf[x], b_buf[x], a_buf[x]);
         }
     }
 
@@ -568,30 +627,38 @@ static bool writeLine(QDataStream &s, QByteArray &buf)
     return (s.status() == QDataStream::Ok);
 }
 
-static bool writeImage1(QImage &img, QDataStream &s, PCXHEADER &header)
+static bool writeImage1(const QImage &image, QDataStream &s, PCXHEADER &header)
 {
-    if (img.format() != QImage::Format_Mono) {
-        img = img.convertToFormat(QImage::Format_Mono);
-    }
-    if (img.isNull() || img.colorCount() < 1) {
+    auto tfmt = image.format();
+    if (tfmt != QImage::Format_Mono)
+        tfmt = QImage::Format_Mono;
+
+    if (image.isNull() || image.colorCount() < 1) {
         return false;
     }
-    auto rgb = img.color(0);
-    auto minIsBlack = (qRed(rgb) + qGreen(rgb) + qBlue(rgb)) / 3 < 127;
+
+    ScanLineConverter scl(tfmt);
+    if (image.height() > 0) {
+        scl.convertedScanLine(image, 0); // required to calculate bytesPerLine
+    }
 
     header.Bpp = 1;
     header.NPlanes = 1;
-    header.BytesPerLine = img.bytesPerLine();
+    header.BytesPerLine = scl.bytesPerLine();
+    if (header.BytesPerLine == 0) {
+        header.BytesPerLine = image.bytesPerLine();
+    }
     if (header.BytesPerLine == 0) {
         return false;
     }
 
     s << header;
 
+    auto rgb = image.color(0);
+    auto minIsBlack = (qRed(rgb) + qGreen(rgb) + qBlue(rgb)) / 3 < 127;
     QByteArray buf(header.BytesPerLine, 0);
-
     for (int y = 0; y < header.height(); ++y) {
-        quint8 *p = img.scanLine(y);
+        auto p = scl.convertedScanLine(image, y);
 
         // Invert as QImage uses reverse palette for monochrome images?
         for (int i = 0; i < header.BytesPerLine; ++i) {
@@ -605,7 +672,7 @@ static bool writeImage1(QImage &img, QDataStream &s, PCXHEADER &header)
     return true;
 }
 
-static bool writeImage4(QImage &img, QDataStream &s, PCXHEADER &header)
+static bool writeImage4(const QImage &image, QDataStream &s, PCXHEADER &header)
 {
     header.Bpp = 1;
     header.NPlanes = 4;
@@ -615,7 +682,7 @@ static bool writeImage4(QImage &img, QDataStream &s, PCXHEADER &header)
     }
 
     for (int i = 0; i < 16; ++i) {
-        header.ColorMap.setColor(i, img.color(i));
+        header.ColorMap.setColor(i, image.color(i));
     }
 
     s << header;
@@ -627,7 +694,7 @@ static bool writeImage4(QImage &img, QDataStream &s, PCXHEADER &header)
     }
 
     for (int y = 0; y < header.height(); ++y) {
-        quint8 *p = img.scanLine(y);
+        auto p = image.constScanLine(y);
 
         for (int i = 0; i < 4; ++i) {
             buf[i].fill(0);
@@ -650,11 +717,23 @@ static bool writeImage4(QImage &img, QDataStream &s, PCXHEADER &header)
     return true;
 }
 
-static bool writeImage8(QImage &img, QDataStream &s, PCXHEADER &header)
+static bool writeImage8(const QImage &image, QDataStream &s, PCXHEADER &header)
 {
+    auto tfmt = image.format();
+    if (tfmt == QImage::Format_Grayscale16)
+        tfmt = QImage::Format_Grayscale8;
+
+    ScanLineConverter scl(tfmt);
+    if (image.height() > 0) {
+        scl.convertedScanLine(image, 0); // required to calculate bytesPerLine
+    }
+
     header.Bpp = 8;
     header.NPlanes = 1;
-    header.BytesPerLine = img.bytesPerLine();
+    header.BytesPerLine = scl.bytesPerLine();
+    if (header.BytesPerLine == 0) {
+        header.BytesPerLine = image.bytesPerLine();
+    }
     if (header.BytesPerLine == 0) {
         return false;
     }
@@ -664,7 +743,7 @@ static bool writeImage8(QImage &img, QDataStream &s, PCXHEADER &header)
     QByteArray buf(header.BytesPerLine, 0);
 
     for (int y = 0; y < header.height(); ++y) {
-        quint8 *p = img.scanLine(y);
+        auto p = scl.convertedScanLine(image, y);
 
         for (int i = 0; i < header.BytesPerLine; ++i) {
             buf[i] = p[i];
@@ -681,26 +760,36 @@ static bool writeImage8(QImage &img, QDataStream &s, PCXHEADER &header)
 
     // Write palette
     for (int i = 0; i < 256; ++i) {
-        s << RGB::from(img.color(i));
+        if (tfmt != QImage::Format_Indexed8)
+            s << RGB::from(qRgb(i, i, i));
+        else
+            s << RGB::from(image.color(i));
     }
 
     return (s.status() == QDataStream::Ok);
 }
 
-static bool writeImage24(QImage &img, QDataStream &s, PCXHEADER &header)
+static bool writeImage24(const QImage &image, QDataStream &s, PCXHEADER &header)
 {
+    auto hasAlpha = image.hasAlphaChannel();
     header.Bpp = 8;
-    header.NPlanes = 3;
+    header.NPlanes = hasAlpha ? 4 : 3;
     header.BytesPerLine = header.width();
     if (header.BytesPerLine == 0) {
         return false;
     }
 
-    if (img.format() != QImage::Format_ARGB32 && img.format() != QImage::Format_RGB32) {
-        img = img.convertToFormat(QImage::Format_RGB32);
+    auto tcs = QColorSpace();
+    auto tfmt = image.format();
+#if QT_VERSION >= QT_VERSION_CHECK(6, 8, 0)
+    auto cs = image.colorSpace();
+    if (cs.isValid() && cs.colorModel() == QColorSpace::ColorModel::Cmyk && tfmt == QImage::Format_CMYK8888) {
+        tcs = QColorSpace(QColorSpace::SRgb);
+        tfmt = QImage::Format_RGB32;
     }
-    if (img.isNull()) {
-        return false;
+#endif
+    if (tfmt != QImage::Format_ARGB32 && tfmt != QImage::Format_RGB32) {
+        tfmt = hasAlpha ? QImage::Format_ARGB32 : QImage::Format_RGB32;
     }
 
     s << header;
@@ -708,15 +797,19 @@ static bool writeImage24(QImage &img, QDataStream &s, PCXHEADER &header)
     QByteArray r_buf(header.width(), 0);
     QByteArray g_buf(header.width(), 0);
     QByteArray b_buf(header.width(), 0);
+    QByteArray a_buf(header.width(), char(0xFF));
 
+    ScanLineConverter scl(tfmt);
+    scl.setTargetColorSpace(tcs);
     for (int y = 0; y < header.height(); ++y) {
-        auto p = (QRgb*)img.scanLine(y);
+        auto p = reinterpret_cast<const QRgb *>(scl.convertedScanLine(image, y));
 
         for (int x = 0; x < header.width(); ++x) {
-            QRgb rgb = *p++;
+            auto &&rgb = p[x];
             r_buf[x] = qRed(rgb);
             g_buf[x] = qGreen(rgb);
             b_buf[x] = qBlue(rgb);
+            a_buf[x] = qAlpha(rgb);
         }
 
         if (!writeLine(s, r_buf)) {
@@ -728,6 +821,9 @@ static bool writeImage24(QImage &img, QDataStream &s, PCXHEADER &header)
         if (!writeLine(s, b_buf)) {
             return false;
         }
+        if (hasAlpha && !writeLine(s, a_buf)) {
+            return false;
+        }
     }
 
     return true;
@@ -781,13 +877,15 @@ bool PCXHandler::read(QImage *outImage)
     QImage img;
     if (header.Bpp == 1 && header.NPlanes == 1) {
         ok = readImage1(img, s, header);
-    } else if (header.Bpp == 1 && header.NPlanes == 4) {
+    } else if (header.Bpp == 1 && (header.NPlanes == 4 || header.NPlanes == 3)) {
         ok = readImage4(img, s, header);
+    } else if (header.Bpp == 2 && header.NPlanes == 1) {
+        ok = readImage2(img, s, header);
     } else if (header.Bpp == 4 && header.NPlanes == 1) {
         ok = readImage4v2(img, s, header);
     } else if (header.Bpp == 8 && header.NPlanes == 1) {
         ok = readImage8(img, s, header);
-    } else if (header.Bpp == 8 && header.NPlanes == 3) {
+    } else if (header.Bpp == 8 && (header.NPlanes == 3 || header.NPlanes == 4)) {
         ok = readImage24(img, s, header);
     }
 
@@ -795,8 +893,14 @@ bool PCXHandler::read(QImage *outImage)
         return false;
     }
 
-    img.setDotsPerMeterX(qRound(header.HDpi / 25.4 * 1000));
-    img.setDotsPerMeterY(qRound(header.YDpi / 25.4 * 1000));
+    auto hres = dpi2ppm(header.HDpi);
+    if (hres > 0) {
+        img.setDotsPerMeterX(hres);
+    }
+    auto vres = dpi2ppm(header.YDpi);
+    if (vres > 0) {
+        img.setDotsPerMeterY(vres);
+    }
     *outImage = img;
     return true;
 }
@@ -806,10 +910,8 @@ bool PCXHandler::write(const QImage &image)
     QDataStream s(device());
     s.setByteOrder(QDataStream::LittleEndian);
 
-    QImage img = image;
-
-    const int w = img.width();
-    const int h = img.height();
+    const int w = image.width();
+    const int h = image.height();
 
     if (w > 65536 || h > 65536) {
         return false;
@@ -824,20 +926,20 @@ bool PCXHandler::write(const QImage &image)
     header.YMin = 0;
     header.XMax = w - 1;
     header.YMax = h - 1;
-    header.HDpi = qRound(image.dotsPerMeterX() * 25.4 / 1000);
-    header.YDpi = qRound(image.dotsPerMeterY() * 25.4 / 1000);
+    header.HDpi = qRoundOrZero_T<quint16>(dppm2dpi(image.dotsPerMeterX()));
+    header.YDpi = qRoundOrZero_T<quint16>(dppm2dpi(image.dotsPerMeterY()));
     header.Reserved = 0;
     header.PaletteInfo = 1;
 
     auto ok = false;
-    if (img.depth() == 1) {
-        ok = writeImage1(img, s, header);
-    } else if (img.depth() == 8 && img.colorCount() <= 16) {
-        ok = writeImage4(img, s, header);
-    } else if (img.depth() == 8) {
-        ok = writeImage8(img, s, header);
-    } else if (img.depth() >= 24) {
-        ok = writeImage24(img, s, header);
+    if (image.depth() == 1) {
+        ok = writeImage1(image, s, header);
+    } else if (image.format() == QImage::Format_Indexed8 && image.colorCount() <= 16) {
+        ok = writeImage4(image, s, header);
+    } else if (image.depth() == 8 || image.format() == QImage::Format_Grayscale16) {
+        ok = writeImage8(image, s, header);
+    } else if (image.depth() >= 16) {
+        ok = writeImage24(image, s, header);
     }
 
     return ok;
@@ -886,7 +988,7 @@ QVariant PCXHandler::option(ImageOption option) const
 bool PCXHandler::canRead(QIODevice *device)
 {
     if (!device) {
-        qWarning("PCXHandler::canRead() called with no device");
+        qCWarning(LOG_PCXPLUGIN) << "PCXHandler::canRead() called with no device";
         return false;
     }
 
